@@ -294,24 +294,34 @@ class B0(IModel):
             # Load best model
             self.__saveModel(self.model, "last.pth")
 
+            if hasattr(self.model, 'cpu'):  
+                self.model = self.model.cpu()
+
+            del self.model
+            self.model = None
+            torch.cuda.empty_cache()
+
             self.Export(path.join("TrainResult", self.ProcessName, "Weights", "last.pth"), eExportType.openvino, settingParams['ImageSize'])
             self.Export(path.join("TrainResult", self.ProcessName, "Weights", "best.pth"), eExportType.openvino, settingParams['ImageSize'])
             
             self.Export(path.join("TrainResult", self.ProcessName, "Weights", "last.pth"), eExportType.onnx, settingParams['ImageSize'])
-            self.Export(path.join("TrainResult", self.ProcessName, "Weights", "best.pth"), eExportType.onnxm, settingParams['ImageSize'])
+            self.Export(path.join("TrainResult", self.ProcessName, "Weights", "best.pth"), eExportType.onnx, settingParams['ImageSize'])
 
-            self.model.eval()
+            
             # Signal training completion
             
             self.IsTraining = False
             self.StopTrainingEvent.set()
 
+
+
         except Exception as ex:
             
             print(f"Exception: {str(ex)}")
             if(self.callbacks != None):
                 self.callbacks("Error", f"Training error: {str(ex)}")
-            
+                print(f"Training error: {str(ex)}")
+                
             self.IsTraining = False
         
     def __saveModel(self, model, modelName):
@@ -398,39 +408,7 @@ class B0(IModel):
             match exportType:
                 case exportType.onnx:
                     
-                    tempModelHM = EfficientNetWithFeatures(tempModel)
-                    pathExportHeatMap = pathExport + "HM.onnx"
-                    
-                    torch.onnx.export(
-                        tempModelHM,
-                        dummy_input,
-                        pathExportHeatMap,
-                        input_names=['input'],
-                        output_names=['output', 'features'],  
-                        
-                        # ========== CRITICAL FIX ==========
-                        dynamic_axes={
-                            'input': {
-                                0: 'batch_size',
-                                2: 'height',     
-                                3: 'width'       
-                            },
-                            'output': {0: 'batch_size'},
-                            'features': {
-                                0: 'batch_size',
-                                2: 'height',      
-                                3: 'width'       
-                            }
-                        },
-                        # ==================================
-                        
-                        opset_version=13
-                    )
-                    
-                    
                     pathExport += ".onnx"
-                    
-                    
                     torch.onnx.export(
                         tempModel,                     # Model to export
                         dummy_input,               # Dummy input for tracing
@@ -443,10 +421,44 @@ class B0(IModel):
                         },
                         opset_version=13           # ONNX opset version (adjust if needed)
                         )
+                    
+                    
+                    tempModelHM = EfficientNetWithFeatures(tempModel)
+                    pathExportHeatMap = pathExport + "HM.onnx"
+                    
+                    torch.onnx.export(
+                        tempModelHM,
+                        dummy_input,
+                        pathExportHeatMap,
+                        input_names=['input'],
+                        output_names=['output', 'features'],  
+                        dynamic_axes={
+                            'input': {
+                              0: 'batch_size',
+                            },
+                            'output': {0: 'batch_size'},
+                            'features': {
+                                0: 'batch_size',
+                            }
+                        },
+                        opset_version=13
+                    )
 
+                    if hasattr(tempModelHM, 'cpu'):  
+                        tempModelHM = tempModelHM.cpu()
 
+                    del tempModelHM
+                    tempModelHM = None
+                    torch.cuda.empty_cache()
 
-                                       
+                    if hasattr(tempModel, 'cpu'):  
+                        tempModel = tempModel.cpu()
+
+                    del tempModel
+                    tempModel = None
+                    torch.cuda.empty_cache()
+                    
+                       
                 case eExportType.openvino:
                         if not OPENVINO_AVAILABLE:
                             print("Warning: OpenVINO is not installed. Install with: pip install MemoLib[openvino]")
@@ -460,13 +472,8 @@ class B0(IModel):
 
                         ov.save_model(ov_model, pathExport)
                         
-                        # ov_model = mo.convert_model(
-                        # tempModel, 
-                        # example_input=dummy_input,
-                        # input_shape=dummy_input.shape()
-                        # )
-                        # ov.save_model(ov_model, pathExport)
-    
+
+        
 
 class B1(B0):
 
